import sys
import time
import uuid
import requests
import json
import _thread
from websocket import create_connection

PRODUCTION = False  # For the Staging environment (test betting, fake money) this should be false
PROJECT_ID = "14"
if PRODUCTION:
    TRADE_HOSTNAME = "api.sharpbet.com/platform-api/v1"
    SPORTBOOK_HOSTNAME = "www.sharpbet.com"
else:
    TRADE_HOSTNAME = "api.dev.sharpbet.com/platform-api/v1"
    SPORTBOOK_HOSTNAME = "test-api.sharpbet.com" 


def login(user, password):
    url = f"https://{TRADE_HOSTNAME}/client:login"

    payload = json.dumps({
        "login": user,
        "password": password
    })
    headers = {
        "x-project-id": PROJECT_ID,
        "device": "sample",
        "Content-Type": "application/json"
    }
    response = requests.request("POST", url, headers=headers, data=payload)
    return response.json()["sessionToken"]


def display_balance(token):
    url = f"https://{TRADE_HOSTNAME}/client"

    payload = {}
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + token
    }
    response = requests.request("GET", url, headers=headers, data=payload)
    print("balance", response.text)


def place_bet(token, ref, outcome, stake):
    url = f"https://{TRADE_HOSTNAME}/bet:place"

    payload = json.dumps({
        "placeBetRequestId": str(ref),
        "betType": "SINGLE",
        "locale": "ENG",
        "currency": "EUR",
        "betAmount": int(stake * 100),
        "outcomes": [{
            "outcomeId": outcome["id"],
            "odds": outcome["odds"]
        }],
        "partnerDetails": {
            "endPunterId": "123456",
        }
    })
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + token
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print("bet:place response", response.text)


def search_bets(token, ref):
    url = f"https://{TRADE_HOSTNAME}/bet:search"

    payload = json.dumps({
        "filter": {
            "placeBetRequestIds": [str(ref)],
            "betStatuses": ["UNSETTLED", "VOID", "LOSE", "WIN"]
        }
    })
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + token
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print("bet search response", response.text)


def list_pending_bets(token):
    url = f"https://{TRADE_HOSTNAME}/bet/request:search"

    payload = json.dumps({
        "paging": {
            "currentPage": 0,
            "itemsPerPage": 10
        }
    })
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + token
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print("pending bet response", response.text)


def listen_bet_updates(token):
    url = f"wss://{TRADE_HOSTNAME}/notifier?projectId={PROJECT_ID}"

    ws = create_connection(url)
    payload = json.dumps({
        "messageType": "SUBSCRIBE",
        "payload": {
            "subscriptionKeys": [{
                "notificationTopic": "AUTHORIZED_CLIENT_UPDATES",
            }],
            "projectId": PROJECT_ID,
            "locale": "ENG",
            "token": "Bearer " + token
        }
    })
    ws.send(payload)

    def print_updates():
        while True:
            print("WS order update:", ws.recv())

    _thread.start_new_thread(print_updates, ())


def get_categories():
    url = f"https://{SPORTBOOK_HOSTNAME}/api/sportsbook/categories?locale=ENG"

    payload = json.dumps({
        "filter": {
            "sportId": 1,
            "sportServices": ["LIVE", "PREMATCH"]
        },
        "paging": {
            "currentPage": 0,
            "itemsPerPage": 100
        },
    })
    headers = {
        "x-project-id": PROJECT_ID,
        "device": "sample",
        "Content-Type": "application/json",
        "Accept": "application/json"
    }
    response = requests.request("POST", url, headers=headers, data=payload)
    return response.json()["data"]


def get_tournaments(category):
    url = f"https://{SPORTBOOK_HOSTNAME}/api/sportsbook/tournaments?locale=ENG"

    payload = json.dumps({
        "filter": {
            "sportId": 1,
            "categoryId": category,
            "sportServices": ["LIVE", "PREMATCH"]
        }
    })
    headers = {
        "x-project-id": PROJECT_ID,
        "device": "sample",
        "Content-Type": "application/json",
        "Accept": "application/json"
    }
    response = requests.request("POST", url, headers=headers, data=payload)
    return response.json()["data"]


def get_events(category_id, tournament_id):
    url = f"https://{SPORTBOOK_HOSTNAME}/api/sportsbook/events?locale=ENG"

    payload = json.dumps({
        "filter": {
            "sportId": 1,
            "categoryId": category_id,
            "tournamentId": tournament_id,
            "onlyMainMarketTypes": True,
            "sportServices": ["LIVE", "PREMATCH"]
        }
    })
    headers = {
        "x-project-id": PROJECT_ID,
        "device": "sample",
        "Content-Type": "application/json",
        "Accept": "application/json"
    }
    response = requests.request("POST", url, headers=headers, data=payload)
    return response.json()["data"]


def listen_odds_updates(tournament_id):
    url = f"wss://{SPORTBOOK_HOSTNAME}/ws/notifier2?pid={PROJECT_ID}"
    ws = create_connection(url)
    payload = json.dumps({
        "messageType": "SUBSCRIBE",
        "payload": {
            "subscriptionKeys": [{
                "notificationTopic": "EXPERIMENTAL_TOURNAMENT",
                "id": tournament_id
            }],
            "projectId": PROJECT_ID,
            "locale": "ENG"
        }
    })
    ws.send(payload)

    def print_updates():
        while True:
            print("WS odds update:", ws.recv())

    _thread.start_new_thread(print_updates, ())


def pick_event():
    # pick the first event
    for cat in get_categories():
        for tour in get_tournaments(cat["id"]):
            data = get_events(cat["id"], tour["id"])
            for tourn in data.values():
                for event in tourn["events"].values():
                    if len(event["marketTypes"]) > 0:
                        return (tour["id"], event)


def display_outcomes(event):
    # display all outcomes with their odds and max stake
    for mt in event["marketTypes"].values():
        for mk in mt["markets"].values():
            for out in mk["outcomes"].values():
                name = out["name"]
                odds = out["odds"]
                meta = out["metadata"]
                if "feed" in meta:
                    stake = meta["feed"]["maxStakeByCurrency"]["EUR"]
                else:
                    stake = "?"
                print(name, odds, stake)


def pick_outcome(event):
    for mt in event["marketTypes"].values():
        for mk in mt["markets"].values():
            for out in mk["outcomes"].values():
                return out


if __name__ == "__main__":
    # fetch category/tournament/event list
    # and pick the first event
    tournament_id, event = pick_event()
    display_outcomes(event)

    listen_odds_updates(tournament_id)

    if len(sys.argv) < 3:
        print("Please provide username/password as argv to enable trading")
        sys.exit(0)

    # login, show balance and subscribe to bet updates
    token = login(sys.argv[1], sys.argv[2])
    display_balance(token)

    listen_bet_updates(token)

    # pick the first outcome and place a bet
    ref = uuid.uuid4()
    print("placing bet", ref)
    outcome = pick_outcome(event)
    place_bet(token, ref, outcome, 1.0)
    list_pending_bets(token)
    search_bets(token, ref)

    # wait for user to terminate the program
    while True:
        try:
            time.sleep(1.0)
        except KeyboardInterrupt:
            sys.exit(0)
